% LABELHORIZONS     Label location of horizon in image collection.
% Stephen Gould <sgould@stanford.edu>

function [imgData] = labelHorizons(imgData, dataDir);

nRows = 5; nCols = 4;
if (nargin < 1), imgData = []; end;
if (nargin < 2), dataDir = '~sgould/scratch/scene/data/images/'; end;

% get all images in directory
if (isempty(imgData)),
    disp('Loading images...');
    imgFiles = dir([dataDir, '/*.jpg']);
    for i = 1:length(imgFiles),
        imgData(end + 1).baseName = imgFiles(i).name(1:end - 4);

        imgName = [dataDir, '/', imgFiles(i).name];
        img = imread(imgName);
        imgData(end).width = size(img, 2);
        imgData(end).height = size(img, 1);
        imgData(end).horizon = imgData(end).height / 2; % default
    end;
end;
nImages = length(imgData);

if (nImages < nRows * nCols),
    nRows = ceil(nImages / nCols);
    nCols = ceil(nImages / nRows);
end;

% event processing loop
nIndex = 1;
button = ' ';
while (button ~= 'q'),
    imgHandles = zeros(nRows * nCols, 1);
    handleIndx = zeros(size(imgHandles));
    for i = 1:(nRows * nCols),
        subplot(nRows, nCols, i);
        imgIndx = mod(nIndex + i - 2, nImages) + 1;
        
        imgName = [dataDir, '/', imgData(imgIndx).baseName, '.jpg'];
        img = imread(imgName); imagesc(img); axis off;
        title(['{\bf ', int2str(imgIndx), '} of ', int2str(nImages), ': ', imgData(imgIndx).baseName]);

        [H, W, C] = size(img);
        line([1, W], imgData(imgIndx).horizon * [1, 1], 'LineWidth', 3, 'Color', [0, 1, 0]);
        
        imgHandles(i) = gca;
        handleIndx(i) = imgIndx;
    end;
    
    [x, y, button] = ginput(1);
    if (button == ' '),
        nIndex = mod(nIndex + nRows * nCols - 1, nImages) + 1;
    elseif (button == 'h'),
        nIndex = 1;
    else
        s = find(imgHandles == gca, 1);
        if (~isempty(s)),
            imgIndx = handleIndx(s(1));
            imgData(imgIndx).horizon = y;
        end;
    end;
end;

close(gcf);

