# MAKE INCLUDE FILE FOR STAIR VISION LIBRARY
# Stephen Gould <sgould@stanford.edu>

#######################################################################
# DO NOT EDIT THIS FILE UNLESS YOU KNOW WHAT YOU ARE DOING. IF YOU
# WANT TO OVERRIDE ANY OF THESE SETTINGS DO SO IN YOUR make.local FILE.
#######################################################################

OS = $(shell uname)
BIN_PATH = $(LASIK_PATH)/bin
#BIN_PATH = $(LASIK_PATH)/devBin
EXT_PATH = $(LASIK_PATH)/external
INC_PATH = $(LASIK_PATH)/include
WX_PATH = $(LASIK_PATH)/external/wxGTK/include
SVL_PATH = $(LASIK_PATH)/svl
CUDA_PATH = $(EXT_PATH)/cuda
CUDA_SDK_PATH = $(EXT_PATH)/cuda_sdk

CCC = g++
NVCC := $(CUDA_PATH)/bin/nvcc

# force 32-bit compile (set to 1 in make.local)
FORCE32BIT = 0
ifeq (,$(MACHTYPE))
  MACHTYPE = $(shell uname -im)
endif
ifeq (,$(findstring x86_64,$(MACHTYPE)))
  FORCE32BIT = 1
endif

# compile and link debug symbols (set to 0 in make.local)
SVL_DEBUG_SYMBOLS = 1

# application build settings (set to 0 or 1 in make.local)
BUILD_MEX_APPS = 0
BUILD_GUI_APPS = 1

# use shared libraries (set to 1 in make.local)
SVL_SHARED_LIBS = 0

# compile cuda .cu files (set to 1 in make.local)
BUILD_CUDA_KERNELS = 0

# platform flags
PLAT_CFLAGS = -D__LINUX__
PLAT_LFLAGS = 

OPENGL_LFLAGS = -lGL -lGLU
JPEG_LFLAGS = -ljpeg

# link to OpenCV should be in external/opencv
OPENCV_CFLAGS = -I$(EXT_PATH)/opencv/include/opencv
OPENCV_LFLAGS = -L$(EXT_PATH)/opencv/lib -lcv -lhighgui -lcxcore -lml

# link to Eigen should be in external/eigen
EIGEN_CFLAGS = -I$(EXT_PATH) -DEIGEN_NO_DEBUG -msse2
EIGEN_LFLAGS = -msse2

# link to wxWidgets should be in external/wxGTK
WXDIR := $(EXT_PATH)/wxGTK
WX_CFLAGS = `$(WXDIR)/buildgtk/wx-config --cflags`
WX_LFLAGS = `$(WXDIR)/buildgtk/wx-config --libs core,base,gl` 

# link to include/boost should be in external/boost_headers
BOOST_CFLAGS = -I$(EXT_PATH)/boost_headers

# initialize extra cflags
EXTRA_CFLAGS = -I$(INC_PATH) -I$(EXT_PATH) -I$(LASIK_PATH)/external/wxGTK/include/ -DGL_GLEXT_PROTOTYPES=1 -DBUILD_CUDA_KERNELS=${BUILD_CUDA_KERNELS}

# cuda flags
OSLOWER = $(shell uname -s 2>/dev/null | tr [:upper:] [:lower:])
CUDA_CFLAGS = -DUSE_CUDA -I$(CUDA_PATH)/include -I$(CUDA_SDK_PATH)/common/inc
CUDA_LFLAGS = -L$(CUDA_SDK_PATH)/lib -lcutil -L$(CUDA_SDK_PATH)/common/lib/$(OSLOWER) \
	-L$(CUDA_PATH)/lib -lcudart

# include local user make.local to override any settings
ifeq ($(wildcard $(LASIK_PATH)/make.local), $(LASIK_PATH)/make.local)
  -include $(LASIK_PATH)/make.local
endif

# extra compiler and linker flags
ifeq ($(USE_OPENGL), 1)
  EXTRA_LFLAGS += $(OPENGL_LFLAGS)
  GRAPHICS_LINK_FLAGS = ${OPENGL_LFLAGS}
endif

ifeq ($(USE_OPENCV), 1)
  EXTRA_CFLAGS += $(OPENCV_CFLAGS)
  EXTRA_LFLAGS += $(OPENCV_LFLAGS) $(JPEG_LFLAGS)
endif

ifeq ($(USE_EIGEN), 1)
  EXTRA_CFLAGS += $(EIGEN_CFLAGS)
  EXTRA_LFLAGS += $(EIGEN_LFLAGS)
endif

ifeq ($(USE_WX), 1)
  EXTRA_CFLAGS += $(WX_CFLAGS)
  EXTRA_LFLAGS += $(WX_LFLAGS)
endif

ifeq ($(USE_BOOST), 1)
  EXTRA_CFLAGS += $(BOOST_CFLAGS)
endif

ifeq ($(BUILD_CUDA_KERNELS), 1)
  EXTRA_CFLAGS += $(CUDA_CFLAGS) 
  EXTRA_LFLAGS += $(CUDA_LFLAGS)  
endif

# SVL libraries
STATICSVL = $(BIN_PATH)/libsvlCuda.a $(BIN_PATH)/libsvlGraphics.a $(BIN_PATH)/libsvlVision.a \
	$(BIN_PATH)/libsvlML.a $(BIN_PATH)/libsvlPGM.a $(BIN_PATH)/libsvlBase.a

SHAREDSVL = $(BIN_PATH)/libsvlCuda.so.2.0.0 $(BIN_PATH)/libsvlGraphics.so.2.0.0 $(BIN_PATH)/libsvlVision.so.2.0.0 \
	$(BIN_PATH)/libsvlML.so.2.0.0 $(BIN_PATH)/libsvlPGM.so.2.0.0 $(BIN_PATH)/libsvlBase.so.2.0.0

EXTLIB = $(EXT_PATH)/xmlParser/xmlParser.a $(EXT_PATH)/lbfgs/lbfgs.a $(EXT_PATH)/libsvm/libsvm.a

ifeq ($(SVL_SHARED_LIBS), 0)
  LIBSVL = $(STATICSVL)
else
  LIBSVL = $(SHAREDSVL)
  EXTRA_LFLAGS += -Wl,-rpath,$(BIN_PATH) -L$(BIN_PATH) \
	-lsvlCuda -lsvlGraphics -lsvlVision -lsvlML -lsvlPGM -lsvlBase
endif

# compiler and linker flags
ifeq ($(strip $(FORCE32BIT)), 0)
  CFLAGS = -O3 -fPIC $(PLAT_CFLAGS) $(EXTRA_CFLAGS) -Wall
  LFLAGS = -lm -lpthread $(PLAT_LFLAGS) $(LIBSVL) $(EXTRA_LFLAGS) $(EXTLIB)
  NVCCFLAGS = -Xcompiler -fPIC $(OPENCV_CFLAGS) $(CUDA_CFLAGS) -I$(INC_PATH)
else
  CFLAGS = -O3 -m32 $(PLAT_CFLAGS) $(EXTRA_CFLAGS) -Wall
  LFLAGS = -lm -m32 -lpthread $(PLAT_LFLAGS) $(LIBSVL) $(EXTRA_LFLAGS) $(EXTLIB)
  NVCCFLAGS = -Xcompiler -fPIC -m32 $(OPENCV_CFLAGS) $(CUDA_CFLAGS) -I$(INC_PATH)
endif

ifeq ($(strip $(SVL_DEBUG_SYMBOLS)), 1)
  CFLAGS += -g
  LFLAGS += -g
endif

OBJ = $(SRC:.cpp=.o)

