% PREPROCESSSUPERPIXELS
% Stephen Gould <sgould@stanford.edu>

INPUT_DIR = '/afs/cs/u/sgould/scratch/scene/data/images/';
OUTPUT_DIR = '/afs/cs/u/sgould/scratch/scene/data/superpixels/';

SEGEXT = '.seg1000';
SEGMENTATION_METHOD = 1; % 0: felzenschwalb, 1: ren-malik
OVERWRITE_EXISTING = 0;

% for Ren-Malik segmentation
if ~exist('cncut')
    addpath('/afs/cs/u/sgould/projects/segmentation/superpixels');
    addpath('/afs/cs/u/sgould/projects/segmentation/superpixels/yu_imncut');
end;

N_sp2 = 1000; N_sp = 200; N_ev = 40;

% for Felzenschwalb segmentation
FELZEXE = '/afs/cs/u/sgould/projects/segmentation/segment/segment -i 0.5 200 20';

% process files
files = dir([INPUT_DIR, '*.jpg']);
%files = dir([INPUT_DIR, '*.png']);
rand('twister', sum(100 * clock));
files = files(randperm(size(files, 1)));
for i = 1:length(files),
    disp(['Processing ', files(i).name, ' (', int2str(i), ' of ', ...
	  int2str(length(files)), ')...']);

    % skip if file exists
    outputFilename = [OUTPUT_DIR, files(i).name(1:end-4), SEGEXT];
    if ((OVERWRITE_EXISTING == 0) && exist(outputFilename, 'file')),
      continue;
    end;

    img = im2double(imread([INPUT_DIR, files(i).name]));
    origN = size(img, 1);
    origM = size(img, 2);

    if ((origN > 320) || (origM > 320)),
        if (origN > origM),
            img = imresize(img, [320, NaN], 'bilinear');
        else
            img = imresize(img, [NaN, 320], 'bilinear');
        end;
    end;
    N = size(img, 1);
    M = size(img, 2);
    img(img < 0) = 0;
    img(img > 1) = 1;
    
    if (size(img, 3) == 1),
        img = repmat(img, [1, 1, 3]);
    end;

    if (SEGMENTATION_METHOD == 0),
        imwrite(img, '/tmp/img.ppm');
        system([FELZEXE, ' /tmp/img.ppm /tmp/img.int']);
        segImg = load('/tmp/img.int');
        figure(1); imagesc(segImg); drawnow;
        disp([max(segImg(:)), length(unique(segImg(:)))]);
    else       
        % ncut parameters for superpixel computation
        diag_length = sqrt(N*N + M*M);
        par = imncut_sp;
        par.int=0;
        par.pb_ic=1;
        par.sig_pb_ic=0.05;
        par.sig_p=ceil(diag_length/50);
        par.verbose=0;
        par.nb_r=ceil(diag_length/60);
        par.rep = -0.005;  % stability?  or proximity?
        par.sample_rate=0.2;
        par.nv = N_ev;
        par.sp = N_sp;

        fprintf('running PB\n');
        [emag,ephase] = pbWrapper(img, par.pb_timing);
        emag = pbThicken(emag);
        par.pb_emag = emag;
        par.pb_ephase = ephase;
        clear emag ephase;

        st = clock;
        fprintf('Ncutting...');
        [Sp, Seg] = imncut_sp(img, par);
        fprintf(' took %.2f minutes\n', etime(clock,st)/60);

        st = clock;
        fprintf('Refining...');
        Sp2 = clusterLocations(Sp, ceil(N*M/N_sp2));
        fprintf(' took %.2f minutes\n', etime(clock,st)/60);

        segImg = Sp2;
    end;

    if ((N ~= origN) || (M ~= origM)),
        segImg = imresize(segImg, [origN, origM], 'nearest');
    end;

    segImg = segImg - min(segImg(:));
    dlmwrite([OUTPUT_DIR, files(i).name(1:end-4), SEGEXT], segImg, 'delimiter', ' ');
end;
