/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    sceneRegionFeatures.h
** AUTHOR(S):   Stephen Gould <sgould@stanford.edu>
** DESCRIPTION:
**  This file encapsulates computation of region (and object) features.
**
*****************************************************************************/

#pragma once

#include <cstdlib>
#include <cassert>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <vector>

#include "svlBase.h"
#include "svlML.h"
#include "svlPGM.h"
#include "svlVision.h"

#include "sceneConfiguration.h"
#include "sceneEntities.h"
#include "sceneAssignment.h"
#include "sceneInstance.h"

using namespace std;

// SceneRegionFeatures -----------------------------------------------------

namespace SceneRegionFeatures {

    // computes entropy of a region
    double regionEntropy(const SceneInstance *instance, int k);

    // computes the normal and deviation for a straight line fit to points
    double lineFit(const vector<CvPoint>& points, Vector2d *normal = NULL);

    // computes \phi_k({i : P_i = k}, v^{hz})
    void regionFeatures(SceneInstance *instance, int regId, vector<double>& f);

    // computes \phi_k({i : R_i = k}, v^{hz})
    void objectFeatures(SceneInstance *instance, int objId, vector<double>& f);

    // computes \phi_k({i : R_i = k}, v^{hz}) for pedestrian
    void pedestrianFeatures(SceneInstance *instance, int objId, vector<double>& f);

    // computes \phi_{kl}({i : P_i = k}, {j : P_j = l})
    void regionPairFeatures(SceneInstance *instance, int k, int l, vector<double>& f);

    // load camera height prior
    void loadCameraHeightPrior(map<double, double>& camHeightPrior);

    double getHeightMarginal(double h, double v, double v0, map<double, double>& camHeightPrior, int imgHeight);
};
