/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    sceneRegionClassifier.h
** AUTHOR(S):   Tianshi Gao <tianshig@stanford.edu>
** DESCRIPTION:
**  Defines an interface for region classifier which contains 
**  feature extractor + classifier
**
*****************************************************************************/

#pragma once

#include "cv.h"
#include "cxcore.h"
#include "ml.h"

#include "Eigen/Core"

#include "svlBase.h"
#include "svlVision.h"
#include "svlML.h"
#include "sceneHOGFeatures.h"
#include "sceneSlidingWindowDetector.h"

using namespace std;

class SceneRegionObjectClassifier {
 protected:
    svlBinaryClassifier *_classifier;
    bool _owns_classifier;

    SceneLinearClassifier *_patchClassifier;
    bool _owns_patchClassifier;

    SceneHOGFeatures* _extractor;
    bool _owns_extractor;
    
    string _objectName;

 public:
    SceneRegionObjectClassifier(const char *name = NULL);
    virtual ~SceneRegionObjectClassifier();

    bool load(const char* modelFilename, const char* extractorFilename, bool bPatchBased = false);
    void loadImage(const char* imgFilname);
    void loadImage(IplImage* img);
    double classifyRegion(const CvMat* regionMask, IplImage* regionImg = NULL);
    double classifyRegion(const CvRect& r, IplImage *regionImg);
    double classifyRegion(IplImage* patchImg);
    int getWinWidth() {SVL_ASSERT(_extractor != NULL); return _extractor->getWinWidth();}
    int getWinHeight() {SVL_ASSERT(_extractor != NULL); return _extractor->getWinHeight();}
    string getObjName() {return _objectName;}

 private:
    void init(const char *name = NULL);
};
