/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    sceneTestMoves.cpp
** AUTHOR(S):   Stephen Gould <sgould@stanford.edu>
** DESCRIPTION:
**  Code for testing pixel and region moves.
**
*****************************************************************************/

#include <cstdlib>
#include <cassert>
#include <iostream>
#include <fstream>
#include <iomanip>

#include "svlBase.h"
#include "svlPGM.h"
#include "svlVision.h"

#include "sceneConfiguration.h"
#include "sceneEntities.h"
#include "sceneAssignment.h"
#include "sceneInstance.h"
#include "sceneModel.h"
#include "sceneRegionModel.h"
#include "sceneObjectModel.h"
#include "sceneVisualization.h"

#define WINDOW_NAME "sceneTestInstance"

using namespace std;

// prototypes ---------------------------------------------------------------

void visualize(SceneInstance &scene);

// main ---------------------------------------------------------------------

void usage()
{
    cerr << SVL_USAGE_HEADER << "\n";
    cerr << "USAGE: ./sceneTestInstance [OPTIONS] <basename>\n";
    cerr << "OPTIONS:\n"
         << "  -x                :: visualize\n"
         << SVL_STANDARD_OPTIONS_USAGE
         << endl;
}

int main(int argc, char *argv[])
{
    captureOpenCVerrors();

    // read commandline parameters
    bool bVisualize = false;

    SVL_BEGIN_CMDLINE_PROCESSING(argc, argv)
        SVL_CMDLINE_BOOL_OPTION("-x", bVisualize)
    SVL_END_CMDLINE_PROCESSING(usage());

    if (SVL_CMDLINE_ARGC != 1) {
        usage();
        return -1;
    }

    svlCodeProfiler::tic(svlCodeProfiler::getHandle("main"));

    const char *baseName = SVL_CMDLINE_ARGV[0];

    string imgFilename = gSceneConfig.filename("imgDir", baseName, "imgExt");
    SceneInstance scene(imgFilename.c_str(), baseName);

    if (bVisualize) {
        visualize(scene);
    }

    ofstream ofs("debug.txt");
    for (int y = 0; y < scene.height(); y++) {
        for (int x = 0; x < scene.width(); x++) {
            ofs << toString(scene.rawPixelFeatures(x, y)) << "\n";
        }
    }
    ofs.close();

    svlCodeProfiler::toc(svlCodeProfiler::getHandle("main"));
    svlCodeProfiler::print(cerr);

    return 0;
}

// helper functions ------------------------------------------------------------------

void visualize(SceneInstance &scene)
{
    IplImage *canvas = cvCloneImage(scene.image());
    
    visualizeRegions(&scene, canvas);
    visualizeBoundaries(&scene, canvas);
    cvNamedWindow(WINDOW_NAME, 1);
    cvShowImage(WINDOW_NAME, canvas);
    cvWaitKey(-1);
    
    cvReleaseImage(&canvas);
}
