% PROCESS MTURK DATA
% Stephen Gould <sgould@stanford.edu>
%

DATADIR = '/afs/cs/u/sgould/scratch2/msrc/data/';
IMGDIR = [DATADIR, 'images/'];
LBLDIR = [DATADIR, 'labels/'];
MTURKDIR = '/afs/cs/u/sgould/www/labeler/data/output/';

% zero-based labels
NAMES = {'building', 'grass', 'tree', 'cow', 'sheep', 'sky', 'airplane', 'water', ...
    'face', 'car', 'bicycle', 'flower', 'sign', 'bird', 'book', 'chair', 'road', ...
    'cat', 'dog', 'body', 'boat'};

% parse images -----------------------------------------------------------

images = dir([IMGDIR, '*.jpg']);
images = {images(:).name};
%images = {images{randperm(length(images))}};

%images = {'1_20_s.jpg'};
%images = {'1_14_s.jpg'};

for i = 1:length(images),
    baseName = images{i}(1:end - 4);
    
    % --- load mturk labels ---
    mturkFiles = dir([MTURKDIR, lower(baseName), '.*.segments.png']);    
    if (length(mturkFiles) == 0), continue; end;
     
    if (length(mturkFiles) > 1),
        [n, indx] = sort([mturkFiles(:).datenum]);
        lblFilename = mturkFiles(indx(end)).name;
    else
        lblFilename = mturkFiles.name;
    end;
    
    img = imread([IMGDIR, baseName, '.jpg']);
    regions = load([LBLDIR, baseName, '.21.regions.txt']);
    layersImg = imread([MTURKDIR, lblFilename]);
    [H, W, C] = size(img);
        
    layers = layersImg(:, :, 1) / 32 +  8 * (layersImg(:, :, 2) / 32 + 8 * layersImg(:, :, 3) / 32);
    layers = medfilt2(layers, [3, 3]);
   
    layerIndx = unique(layers(:));
    if (strcmp(baseName, '1_10_s')),
        maxLayerIndx = max(layerIndx);
        for j = 1:length(layerIndx),
            [L, n] = bwlabel(layers == layerIndx(j));
            for k = 2:n,
                layers(L == k) = maxLayerIndx + 1;
                maxLayerIndx = maxLayerIndx + 1;
            end;
        end;
        layerIndx = unique(layers(:));
    end;
    
    bboxes = [];
    for j = 1:length(layerIndx),
        indx = find(layers == layerIndx(j));
        [y, x] = ind2sub([H, W], indx);
        lbl = mode(regions(indx));
        if (lbl < 0), continue; end;
        lbl = lbl + 1;
        if (strcmp(NAMES{lbl}, 'cow') || strcmp(NAMES{lbl}, 'sheep')),
            bboxes = [bboxes; lbl, min(x), min(y), max(x) - min(x) + 1, max(y) - min(y) + 1];
        end;        
    end;
    
    subplot(2, 1, 1); imagesc(img); axis off;
    for j = 1:size(bboxes, 1),
        rectangle('Position', bboxes(j, 2:end), 'LineWidth', 4, 'EdgeColor', [1, 0, 0]);
    end;
    subplot(2, 1, 2); imagesc(layersImg); axis off;
    drawnow; pause(1);
    
    % --- save ---
    if (1),
        fid = fopen([LBLDIR, baseName, '.objects.txt'], 'wt');
        for j = 1:size(bboxes, 1),
            fprintf(fid, '%s %d %d %d %d\n', NAMES{bboxes(j, 1)}, bboxes(j, 2:end));
        end;        
        fclose(fid);
    end;    
end;

% show images still needing labels
%disp(needLabels);

