#!/bin/csh
# EVALUATE PIXEL MODEL
# Stephen Gould <sgould@stanford.edu>
#

set BASEDIR = "/afs/cs/u/tianshig/SVL/stanford/sceneUnderstand/experiment/"
cd ${BASEDIR}

set FLAGS = "NOQSUB"

if ((`hostname` == "ai.Stanford.EDU") || (`hostname` == "robo.Stanford.EDU")) then
    #set FLAGS = "-q quicksail"
    set FLAGS = "-l arch=x86_64"
endif

set BINDIR = "/afs/cs/u/tianshig/SVL/bin"

set FEATURESDIR = "${BASEDIR}/features/"
set MODELDIR = "${BASEDIR}/models/"

set TRAINLIST = "${BASEDIR}/trainList.txt"
set EVALLIST = "${BASEDIR}/evalList.txt"
set ALLLIST = "${BASEDIR}/allList.txt"

set CONFIG = "${BASEDIR}/defaultConfig.xml"

# evaluate
set OPTIONS = "-outClasses .pixelClasses.txt -outImages .pixelResults.png"
set OPTIONS = "${OPTIONS} -verbose -profile"

@ counter = 0
foreach i (`cat $EVALLIST | sort`)
    set CMD = "$BINDIR/sceneEvalPixelModel -config ${CONFIG} $OPTIONS $i"
    echo $CMD

    if (! -e ${BASEDIR}/output/${i}.pixelClasses.txt) then
        if ("$FLAGS" == "NOQSUB") then
            $CMD
        else
            set SCRIPT = "scripts/__evalPixelModel.${i}.sh"
            echo "cd ${BASEDIR}/scripts" > $SCRIPT
            echo "${CMD}" >>  $SCRIPT

            if ($counter % 5 == 0) then
                waitforQs.sh 50
                sleep 1
            endif
            qsub $FLAGS $SCRIPT
            @ counter += 1
        endif
    endif
end

# process training
set OPTIONS = ""

foreach i (`cat $TRAINLIST | sort`)
    set CMD = "$BINDIR/sceneEvalPixelModel -config ${CONFIG} $OPTIONS $i"
    echo $CMD

    if (! -e ${BASEDIR}/features/${i}.features.bin) then
        if ($FLAGS == "NOQSUB") then
            $CMD
        else
            set SCRIPT = "scripts/__evalPixelModel.${i}.sh"
            echo "cd ${BASEDIR}/scripts" > $SCRIPT
            echo "${CMD}" >>  $SCRIPT

            if ($counter % 5 == 0) then
                waitforQs.sh 50
                sleep 1
            endif
            qsub $FLAGS $SCRIPT
            @ counter += 1
        endif
    endif
end
