#!/bin/csh
# EVALUATE REGION MODEL
# Stephen Gould <sgould@stanford.edu>
#
# USAGE: ./evalRegionModel.sh (<clear>) (<train>)
#

set BASEDIR = "/afs/cs/u/tianshig/SVL/stanford/sceneUnderstand/experiment/"
cd ${BASEDIR}

set FLAGS = "NOQSUB"

if ((`hostname` == "ai.Stanford.EDU") || (`hostname` == "robo.Stanford.EDU") || (`hostname` == "scail.Stanford.EDU")) then
    #set FLAGS = "-q quicksail"
    set FLAGS = "-l arch=x86_64"
#    set FLAGS = "-q daglab"
endif

set BINDIR = "/afs/cs/u/tianshig/SVL/bin"

set FEATURESDIR = "${BASEDIR}/features/"
set MODELDIR = "${BASEDIR}/models/"
set OUTPUTDIR = "${BASEDIR}/output/"

set TRAINLIST = "${BASEDIR}/trainList.txt"
#set EVALLIST = "${BASEDIR}/evalList.txt"
set ALLLIST = "${BASEDIR}/allList.txt"
set EVALLIST = "${BASEDIR}/allList.txt"
#set EVALLIST = "${BASEDIR}/trainList_tmp.txt"
set CONFIG = "${BASEDIR}/defaultConfig.xml"
#set CONFIG = "${BASEDIR}/msrcPixelModelConfig.xml"
set LIST = $EVALLIST
if ($2 == 0) then
    set LIST = $EVALLIST
endif
if ($2 == 1) then
    set LIST = $TRAINLIST
endif

if ($1 == 1) then
    if ($LIST == $EVALLIST) then
        rm -f ${OUTPUTDIR}/*.predictedRegions.txt
        rm -f ${OUTPUTDIR}/*.predictedClasses.txt
        rm -f ${OUTPUTDIR}/*.predictedHorizon.txt
        rm -f ${OUTPUTDIR}/*.predictedLabels.txt
        rm -f ${OUTPUTDIR}/*.results.png
    else
        rm -f ${OUTPUTDIR}/*.mistakes.bin
    endif
endif

# evaluate on either training or testing sets
set OPTIONS = "-m REGION"
if ($LIST == $EVALLIST) then
    set OPTIONS = "${OPTIONS} -outRegions .predictedRegions.txt"
    set OPTIONS = "${OPTIONS} -outClasses .predictedClasses.txt"
    set OPTIONS = "${OPTIONS} -outHorizon .predictedHorizon.txt"
    set OPTIONS = "${OPTIONS} -outImages .results.png"
else
#    set OPTIONS = "${OPTIONS} -outRegions .predictedRegions.txt"
#    set OPTIONS = "${OPTIONS} -outLabels .predictedLabels.txt"
    set OPTIONS = "${OPTIONS} -set svlBase.svlLogger logLevel MESSAGE"
endif

@ counter = 0
foreach i (`cat $LIST | sort`)
    if ($LIST == $EVALLIST) then
        set CMD = "$BINDIR/sceneInference -config ${CONFIG} $OPTIONS $i"
        set FILECHECK = "${OUTPUTDIR}/${i}.predictedRegions.txt"
    else
        set CMD = "$BINDIR/sceneInference -config ${CONFIG} -record ${OUTPUTDIR}/${i}.mistakes.bin $OPTIONS $i"
        set FILECHECK = "${OUTPUTDIR}/${i}.mistakes.bin"
    endif
    echo $CMD

    if (! -e $FILECHECK) then
        if ("$FLAGS" == "NOQSUB") then
            $CMD
        else
            set SCRIPT = "scripts/__evalRegionModel.${i}.sh"
            echo "cd ${BASEDIR}/scripts" > $SCRIPT
            echo "${CMD}" >> $SCRIPT

            if ($counter % 5 == 0) then
                waitforQs.sh 10
                sleep 1
            endif
            qsub $FLAGS $SCRIPT
            @ counter += 1
        endif
    endif
end

