#!/bin/csh
# EVALUATE REGION MODEL WITH LP INFERENCE
# Stephen Gould <sgould@stanford.edu>
#
# USAGE: ./evalRegionModelLP.sh (<clear>) (<train>)
#

set BASEDIR = "/afs/cs/u/tianshig/SVL/stanford/sceneUnderstandPascal/experiment_stanford/"
cd ${BASEDIR}

set FLAGS = "NOQSUB"

if ((`hostname` == "ai.Stanford.EDU") || (`hostname` == "robo.Stanford.EDU")) then
    #set FLAGS = "-q quicksail"
    set FLAGS = "-l arch=x86_64"
endif

set BINDIR = "/afs/cs/u/tianshig/SVL/bin"

set FEATURESDIR = "${BASEDIR}/features/"
set MODELDIR = "${BASEDIR}/models/"
set OUTPUTDIR = "${BASEDIR}/output/"

set TRAINLIST = "${BASEDIR}/trainList.txt"
set EVALLIST = "${BASEDIR}/evalList.txt"
set ALLLIST = "${BASEDIR}/allList.txt"

set CONFIG = "${BASEDIR}/defaultConfigLP.xml"

set LIST = $EVALLIST
if ($2 == 0) then
    set LIST = $EVALLIST
endif
if ($2 == 1) then
    set LIST = $TRAINLIST
endif

if ($1 == 1) then
    if ($LIST == $EVALLIST) then
        rm -f ${OUTPUTDIR}/*.predictedRegionsLP.txt
        rm -f ${OUTPUTDIR}/*.predictedClassesLP.txt
        rm -f ${OUTPUTDIR}/*.predictedHorizonLP.txt
        rm -f ${OUTPUTDIR}/*.predictedLabelsLP.txt
        rm -f ${OUTPUTDIR}/*.resultsLP.png
    else
        rm -f ${OUTPUTDIR}/*.mistakes.bin
    endif
endif

# evaluate on either training or testing sets
set OPTIONS = "-m REGION"
if ($LIST == $EVALLIST) then
    set OPTIONS = "${OPTIONS} -outRegions .predictedRegionsLP.txt"
    set OPTIONS = "${OPTIONS} -outClasses .predictedClassesLP.txt"
    set OPTIONS = "${OPTIONS} -outHorizon .predictedHorizonLP.txt"
    set OPTIONS = "${OPTIONS} -outImages .resultsLP.png"
    set OPTIONS = "${OPTIONS} -maxIterations 10"
    set OPTIONS = "${OPTIONS} -maxDDIterations 1000"
else
#    set OPTIONS = "${OPTIONS} -outRegions .predictedRegionsLP.txt"
#    set OPTIONS = "${OPTIONS} -outLabels .predictedLabelsLP.txt"
    set OPTIONS = "${OPTIONS} -set svlBase.svlLogger logLevel MESSAGE"
endif

@ counter = 0
foreach i (`cat $LIST | sort`)
    if ($LIST == $EVALLIST) then
        set CMD = "$BINDIR/sceneInferenceLP -config ${CONFIG} $OPTIONS $i"
        set FILECHECK = "${OUTPUTDIR}/${i}.predictedRegionsLP.txt"
    else
        set CMD = "$BINDIR/sceneInferenceLP -config ${CONFIG} -record ${OUTPUTDIR}/${i}.mistakes.bin $OPTIONS $i"
        set FILECHECK = "${OUTPUTDIR}/${i}.mistakes.bin"
    endif
    echo $CMD

    if (! -e $FILECHECK) then
        if ("$FLAGS" == "NOQSUB") then
            $CMD
        else
            set SCRIPT = "scripts/__evalRegionModel.${i}.sh"
            echo "cd ${BASEDIR}/scripts" > $SCRIPT
            echo "${CMD}" >> $SCRIPT

            if ($counter % 5 == 0) then
                waitforQs.sh 50
                sleep 1
            endif
            qsub $FLAGS $SCRIPT
            @ counter += 1
        endif
    endif
end

