/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    sceneVisualization.h
** AUTHOR(S):   Stephen Gould <sgould@stanford.edu>
** DESCRIPTION:
**  Various visualizations for scenes.
**
*****************************************************************************/

#pragma once

#include <cstdlib>
#include <cassert>

#include "Eigen/Core"

#include "svlBase.h"
#include "svlML.h"
#include "svlPGM.h"
#include "svlVision.h"

#include "sceneConfiguration.h"
#include "sceneEntities.h"
#include "sceneAssignment.h"
#include "sceneInstance.h"

using namespace std;
USING_PART_OF_NAMESPACE_EIGEN

// visualization routines
IplImage *visualize(const SceneInstance *scene, bool bBlackOutSingletonRegions = false);

void visualizeClasses(const SceneInstance *scene, IplImage *canvas, double alpha = 0.5);

void visualizeObjects(const SceneInstance *scene, IplImage *canvas, int thickness = 1);

void visualizeRegions(const SceneInstance *scene, IplImage *canvas, bool bAvgColour = true,
    double alpha = 0.5);

void visualizeHorizon(const SceneInstance *scene, IplImage *canvas, int thickness = 3);

void visualizeContrast(const SceneInstance *scene, IplImage *canvas, double alpha = 1.0);

void visualizeBoundaries(const SceneInstance *scene, IplImage *canvas,
    unsigned char red = 0xff, unsigned char green = 0xff, unsigned char blue = 0xff);

void visualizeClassBoundaries(const SceneInstance *scene, IplImage *canvas, bool bFancy = false);

void visualizeSegmentation(const SceneInstance *scene, IplImage *canvas, 
    const CvMat *seg, int activeSeg = -1, unsigned char red = 0xff, 
    unsigned char green = 0x00, unsigned char blue = 0x00,
    unsigned char activeRed = 0x00, unsigned char activeGreen = 0xff,
    unsigned char activeBlue = 0x00);

void maskSingletonRegions(const SceneInstance *scene, IplImage *canvas,
    unsigned char red = 0xff, unsigned char green = 0x00, unsigned char blue = 0x00);

void maskNonMatchingRegions(const SceneInstance *scene, IplImage *canvas, int regionId,
    unsigned char red = 0xff, unsigned char green = 0x00, unsigned char blue = 0x00);
void maskNonMatchingRegions(IplImage *canvas, const CvMat *mask, int regionId,
    unsigned char red = 0xff, unsigned char green = 0x00, unsigned char blue = 0x00);

IplImage *visualizeRegionArray(const SceneInstance *scene, int minSize = 0, int classId = -1);

IplImage *visualizeRegionSingletons(const SceneInstance *scene);

IplImage *visualizePixelEntropy(const SceneInstance *scene);

IplImage *visualizeOverSegmentation(const CvMat *seg);

IplImage *visualizeDepths(const MatrixXd &depths, double maxDepth = -1.0);

